# Django settings for cms project.
import os
PROJECT_DIR = os.path.dirname(__file__)

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = 'cms.sqlite'

SITE_ID = 1

MEDIA_ROOT = os.path.join(PROJECT_DIR, 'media/')

MEDIA_URL = '/media/'

ADMIN_MEDIA_PREFIX = '/media/admin/'

SECRET_KEY = '*xq7m@)*f2awoj!spa0(jibsrz9%c0d=e(g)v*!17y(vx0ue_3'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.auth",
    "django.core.context_processors.i18n",
    "django.core.context_processors.debug",
    "django.core.context_processors.request",
    "django.core.context_processors.media",
    'django.core.context_processors.csrf',
)



ROOT_URLCONF = 'testapp.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'django.contrib.sites',
    'localcrawler',
    'testapp',
)

gettext = lambda s: s

LANGUAGE_CODE = "en"

APPEND_SLASH = True


from django.conf import settings

__all__ = ['FAIL_ON_BAD_SOUP', 'CHECK_MEDIA', 'CHECK_JS', 'CHECK_CSS', 'ENTRY_POINT']

DEFAULTS = {
    'FAIL_ON_BAD_SOUP': True,
    'CHECK_MEDIA': True,
    'CHECK_JS': True,
    'CHECK_CSS': True,
    'ENTRY_POINT': '/',
}

for key, value in DEFAULTS.items():
    globals()[key] = getattr(settings, 'LOCALCRAWLER_%s' % key, value)


from django.conf import settings
from django.conf.urls.defaults import handler500, handler404, patterns, include, \
    url
from django.contrib import admin
from testapp.views import RenderTemplate

admin.autodiscover()

urlpatterns = patterns('',
    (r'^$', RenderTemplate('index.html')),
    (r'(?P<template>.+?)/$', RenderTemplate()),
    (r'^admin/', include(admin.site.urls)),
    url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT, 'show_indexes': True})
)


from django.test import TestCase
from localcrawler import settings
from localcrawler.core import Crawler


class LocalcrawlerTestCase(TestCase):
    def test_localcrawler(self):
        crawler = Crawler(settings.ENTRY_POINT, settings.CHECK_MEDIA,
                          settings.CHECK_CSS, settings.CHECK_JS,
                          settings.FAIL_ON_BAD_SOUP, self.client)
        success = crawler.crawl()
        self.assertTrue(success, "At least one URL failed to load")


from django.core.management.base import BaseCommand
from ...core import Crawler
import sys
import time


class Command(BaseCommand):
    def handle(self, *args, **options):
        crawler = Crawler()
        start = time.time()
        crawler.crawl()
        end = time.time()
        duration = end - start
        print ""
        print "--------------------------"
        print "django-localcrawler report"
        print "--------------------------"
        print ""
        print "Entry Point: %s" % crawler.entry_point
        print "Duration:    %.1fs"  % duration
        print "Crawled:     %s" % crawler.crawled
        print "Succeeded:   %s" % crawler.succeeded
        print "Failed:      %s" % crawler.failed
        print "Success:     %s" % crawler.success
        print ""
        return crawler.success


from django.shortcuts import render_to_response
from django.template.context import RequestContext


class RenderTemplate(object):
    def __init__(self, template=None):
        self.template = template
        
    def __call__(self, request, **kwargs):
        template = self.template or '%s.html' % kwargs['template']
        return render_to_response(template, {}, RequestContext(request))


from BeautifulSoup import BeautifulSoup
from django.test.client import Client
import sys

__all__ = ['Crawler']

class Crawler(object):
    def __init__(self, entry_point='/', media=True, css=True, js=True,
                 bad_soup=True, client=None, output=sys.stderr):
        self.queue = [entry_point]
        self.ignore = []
        self.media = media
        self.css = css
        self.js = js
        self.bad_soup = bad_soup
        if client:
            self.client = client
        else:
            self.client = Client()
        self.output = output
        self.success = True
        self.crawled = 0
        self.failed = 0
        self.succeeded = 0
        
    def crawl(self):
        while self.queue:
            self.check(self.queue.pop(0))
        return self.success
            
    def check(self, url):
        """
        Open a single URL and check it's status code.
        
        If status is OK, run a scan if content type is html.
        """
        response = self.client.get(url, follow=True)
        self.ignore.append(url)
        # check if we're a 200
        if response.status_code != 200:
            self.success = False
            self.report(response.status_code, url, "URL Failed")
            return
        self.succeeded += 1
        html = response.content
        if response.get('Content-Type', '').startswith('text/html'):
            self.scan(html, url)
            
    def report(self, prefix, url, message):
        self.failed += 1
        print >>self.output, "[%s] %s (%s)" % (prefix, url, message)
            
    def scan(self, html, url):
        """
        Scan a HTML document for further links we might be interested in.
        """
        try:
            soup = BeautifulSoup(html)
        except Exception, e:
            if self.bad_soup:
                self.success = False
                self.report("SOUP", url, unicode(e))
            return
        if self.media:
            for img in soup.findAll('img'):
                src = img.get('src', None)
                if self._relevant(src):
                    self.queue.append(src)
        if self.js:
            for js in soup.findAll('script', attrs={'type': 'text/javascript'}):
                src = js.get('src', None)
                if self._relevant(src):
                    self.queue.append(src)
        if self.css:
            for css in soup.findAll('link', attrs={'type': 'text/css'}):
                href = css.get('href', None)
                if self._relevant(href):
                    self.queue.append(href)
        for a in soup.findAll('a'):
            href = a.get('href', None)
            if self._relevant(href):
                self.queue.append(href)
        
    def _relevant(self, url):
        return url and url.startswith('/') and url not in self.ignore


try:
    from cStringIo import StringIO
except ImportError:
    from StringIO import StringIO
from django.core.management import call_command
from django.test import TestCase
from django.test.simple import run_tests
from django.test.utils import teardown_test_environment, setup_test_environment
from localcrawler.core import Crawler
from localcrawler.tests import LocalcrawlerTestCase

class LocalcrawlerAppTestCase(TestCase):
    def assertSio(self, sio, *lines):
        siolines = [line for line in sio.getvalue().split('\n') if line]
        self.assertEqual(siolines, list(lines))
        
    def _test_type(self, name, ext=None):
        ext = ext or name
        crawler = Crawler('/fail_%s/' % name, **{name: False})
        self.assertTrue(crawler.crawl())
        self.assertEqual(crawler.failed, 0)
        self.assertEqual(crawler.succeeded, 1)
        sio = StringIO()
        crawler = Crawler('/fail_%s/' % name, output=sio, **{name: True})
        self.assertFalse(crawler.crawl())
        self.assertEqual(crawler.failed, 1)
        self.assertEqual(crawler.succeeded, 1)
        self.assertSio(sio, "[404] /media/does_not_exist.%s (URL Failed)" % ext)
        crawler = Crawler('/win_%s/' % name, **{name: True})
        self.assertTrue(crawler.crawl())
        self.assertEqual(crawler.failed, 0)
        self.assertEqual(crawler.succeeded, 2)
        crawler = Crawler('/win_%s/' % name, **{name: False})
        self.assertTrue(crawler.crawl())
        self.assertEqual(crawler.failed, 0)
        self.assertEqual(crawler.succeeded, 1)
                
    def test_01_full(self):
        self._fixture_teardown()
        teardown_test_environment()
        call_command('test', 'localcrawler')
        call_command('localcrawler')
        setup_test_environment()
        self._fixture_setup()
    
    def test_02_css(self):
        self._test_type('css')
        
    def test_03_media(self):
        self._test_type('media', 'jpg')
        
    def test_04_js(self):
        self._test_type('js')
        
    def test_05_bad_soup(self):
        crawler = Crawler('/fail_soup/', bad_soup=False)
        self.assertTrue(crawler.crawl())
        self.assertEqual(crawler.failed, 0)
        self.assertEqual(crawler.succeeded, 1)
        sio = StringIO()
        crawler = Crawler('/fail_soup/', bad_soup=True, output=sio)
        self.assertFalse(crawler.crawl())
        self.assertEqual(crawler.failed, 1)
        self.assertEqual(crawler.succeeded, 1)
        self.assertSio(sio, "[SOUP] /fail_soup/ (bad end tag: u\"</scr' + 'ipt>\", at line 11, column 88)")
        
    def test_06_links(self):
        crawler = Crawler('/first/')
        self.assertTrue(crawler.crawl())
        self.assertEqual(crawler.failed, 0)
        self.assertEqual(crawler.succeeded, 2)


from setuptools import setup, find_packages
from localcrawler import __version__
 
setup(
    name='django-localcrawler',
    version=__version__,
    description="django-localcrawler",
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
    keywords='testing,django',
    author='Jonas Obrist',
    author_email='jonas.obrist@divio.ch',
    url='http://github.com/ojii/django-localcrawler',
    license='BSD',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Django>=1.2",
        "BeautifulSoup>=3.0.8.1",
    ]
)